==> atlassian-user-1.9-sources.jar: com/atlassian/user/impl/delegation/DelegatingGroupManagerTemplate.java <==
package com.atlassian.user.impl.delegation;

import com.atlassian.user.*;
import com.atlassian.user.impl.delegation.repository.DelegatingRepository;
import com.atlassian.user.repository.RepositoryIdentifier;
import com.atlassian.user.search.page.Pager;
import com.atlassian.user.search.page.PagerFactory;
import com.atlassian.user.search.page.DefaultPager;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;

public abstract class DelegatingGroupManagerTemplate implements GroupManager
{
    /**
     * Returns the first group manager in iteration order which returns a non-null value from
     * {@link GroupManager#getGroup(String)}, or the last group manager in the iteration
     * order if no group manager meets this criteria.
     *
     * @throws EntityException if any of the delegating managers throws an exception
     * in {@link GroupManager#getGroup(String)}.
     * @throws NullPointerException if the group is null
     */
    protected GroupManager getMatchingGroupManager(Group group) throws EntityException
    {
        GroupManager groupManager = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            groupManager = (GroupManager) iter.next();
            Group foundGroup = groupManager.getGroup(group.getName());

            if (foundGroup != null)
                return groupManager;
        }

        // returns the last groupManager if none matched
        return groupManager;
    }

    public Pager getGroups() throws EntityException
    {
        List groups = new ArrayList();

        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            groups.add(groupManager.getGroups());
        }

        return PagerFactory.getPager(groups);
    }

    public Pager getGroups(User user) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
		List pagers = new LinkedList();

		while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            Pager groupPager = groupManager.getGroups(user);
			pagers.add(groupPager);
        }

		return PagerFactory.getPager(pagers);
    }

    public Pager getMemberNames(Group group) throws EntityException
    {
        GroupManager groupManager = getManagerForGroup(group);

        if (groupManager == null)
            return new DefaultPager();
        else
            return groupManager.getMemberNames(group);
    }

    public Pager getLocalMemberNames(Group group) throws EntityException
    {
        Pager result = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            result = groupManager.getLocalMemberNames(group);

            if (result != null)
                break;
        }

        return result;
    }

    public Pager getExternalMemberNames(Group group) throws EntityException
    {
        Pager result = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.supportsExternalMembership())
            {
                result = groupManager.getExternalMemberNames(group);
            }

            if (result != null)
                break;
        }

        return result;
    }

    public Group getGroup(String groupName) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
        Group foundGroup;

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            foundGroup = groupManager.getGroup(groupName);

			if (foundGroup != null)
                return foundGroup;
        }

        return null;
    }

    public Group createGroup(String groupName) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
        Group createdGroup = null;

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.isCreative())
                createdGroup = groupManager.createGroup(groupName);

            if (createdGroup != null)
                return createdGroup;
        }

        return null;
    }

    /**
     * Removes the specified group, if it is present.
     *
     * @throws EntityException
     *          - representing the exception which prohibited removal
     */
    public void removeGroup(Group group) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            if (groupManager.getGroup(group.getName()) == null)
                continue;

            if (groupManager.isReadOnly(group))
                continue;

            groupManager.removeGroup(group);
            break;
        }
    }

    public void addMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            throw new IllegalArgumentException("Can't add membership for null group");
        GroupManager groupManager = getMatchingGroupManager(group);
        groupManager.addMembership(group, user);
    }

    /**
     * Check whether this manager has a record of membership between the
     * argued user and group.
     *
     * @throws com.atlassian.user.impl.RepositoryException
     *
     */
    public boolean hasMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            return false;
        GroupManager groupManager = getMatchingGroupManager(group);
        return groupManager.hasMembership(group, user);
    }

    public void removeMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            throw new IllegalArgumentException("Can't remove membership for null group");
        GroupManager groupManager = getMatchingGroupManager(group);
        groupManager.removeMembership(group, user);
    }

    /**
     * @return true if users from other systems can be granted membership in the underlying {@link com.atlassian.user.repository.RepositoryIdentifier},
     *         otherwise false.
     * @throws EntityException
     *
     */
    public boolean supportsExternalMembership() throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.supportsExternalMembership())
                return true;
        }

        return false;
    }

    public boolean isReadOnly(Group group) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.getGroup(group.getName()) != null)
                return groupManager.isReadOnly(group);
        }

        return false;
    }

    /**
     * @return the {@link com.atlassian.user.repository.RepositoryIdentifier} which is managed by this instance.
     */
    public RepositoryIdentifier getIdentifier()
    {
        ArrayList repositories = new ArrayList();

        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            RepositoryIdentifier repo = groupManager.getIdentifier();
            repositories.add(repo);
        }

        return new DelegatingRepository(repositories);
    }

    /**
     * Don't use for now. This is very expensive for large LDAP instances. todo: get user/group entities to store which repository they came from
     *
     * @return the {@link RepositoryIdentifier} in which the entity is stored, otherwise null.
     * @throws EntityException
     *
     */
    public RepositoryIdentifier getRepository(Entity entity) throws EntityException
    {
        if (!(entity instanceof Group))
            return null;

        GroupManager groupManager = getMatchingGroupManager((Group) entity);

        return groupManager.getIdentifier();
    }

    /**
     * Used to detemine whether an entity can be created (eg, can call {@link UserManager#createUser(String)} or
     * {@link GroupManager#createGroup(String)}
     *
     * @return true to indicate that {@link Entity} objects can be created by this manager, or false to indicate
     *         not.
     */
    public boolean isCreative()
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.isCreative())
                return true;
        }
        return false;
    }

    /**
     * Returns an iterator of {@link GroupManager} objects, in the order they
     * are to be used for this delegating manager.
     */
    protected abstract Iterator getGroupManagersIterator();

    private GroupManager getManagerForGroup(Group group) throws EntityException
    {
        for (Iterator it = getGroupManagersIterator(); it.hasNext();)
        {
            GroupManager groupManager = (GroupManager) it.next();
            if (groupManager.getGroup(group.getName()) != null)
                return groupManager;
        }

        return null;
    }
}

==> atlassian-user-1.26-sources.jar: com/atlassian/user/impl/delegation/DelegatingGroupManagerTemplate.java <==
package com.atlassian.user.impl.delegation;

import com.atlassian.user.*;
import com.atlassian.user.impl.delegation.repository.DelegatingRepository;
import com.atlassian.user.repository.RepositoryIdentifier;
import com.atlassian.user.search.page.Pager;
import com.atlassian.user.search.page.PagerFactory;
import com.atlassian.user.search.page.DefaultPager;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;

public abstract class DelegatingGroupManagerTemplate implements GroupManager
{
    /**
     * Returns the first group manager in iteration order which returns a non-null value from
     * {@link GroupManager#getGroup(String)}, or the last group manager in the iteration
     * order if no group manager meets this criteria.
     *
     * @throws EntityException if any of the delegating managers throws an exception
     * in {@link GroupManager#getGroup(String)}.
     * @throws NullPointerException if the group is null
     */
    protected GroupManager getMatchingGroupManager(Group group) throws EntityException
    {
        GroupManager groupManager = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            groupManager = (GroupManager) iter.next();
            Group foundGroup = groupManager.getGroup(group.getName());

            if (foundGroup != null)
                return groupManager;
        }

        // returns the last groupManager if none matched
        return groupManager;
    }

    public Pager getGroups() throws EntityException
    {
        List groups = new ArrayList();

        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            groups.add(groupManager.getGroups());
        }

        return PagerFactory.getPager(groups);
    }

    public Pager getGroups(User user) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
		List pagers = new LinkedList();

		while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            Pager groupPager = groupManager.getGroups(user);
			pagers.add(groupPager);
        }

		return PagerFactory.getPager(pagers);
    }

    public List /*<Group>*/ getWritableGroups()
    {
        List groups = new ArrayList();

        for (Iterator it = getGroupManagersIterator(); it.hasNext();)
        {
            GroupManager groupManager = (GroupManager) it.next();
            groups.addAll(groupManager.getWritableGroups());
        }

        return groups;
    }

    public Pager getMemberNames(Group group) throws EntityException
    {
        GroupManager groupManager = getManagerForGroup(group);

        if (groupManager == null)
            return new DefaultPager();
        else
            return groupManager.getMemberNames(group);
    }

    public Pager getLocalMemberNames(Group group) throws EntityException
    {
        Pager result = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            result = groupManager.getLocalMemberNames(group);

            if (result != null)
                break;
        }

        return result;
    }

    public Pager getExternalMemberNames(Group group) throws EntityException
    {
        Pager result = null;
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.supportsExternalMembership())
            {
                result = groupManager.getExternalMemberNames(group);
            }

            if (result != null)
                break;
        }

        return result;
    }

    public Group getGroup(String groupName) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
        Group foundGroup;

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            foundGroup = groupManager.getGroup(groupName);

			if (foundGroup != null)
                return foundGroup;
        }

        return null;
    }

    public Group createGroup(String groupName) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();
        Group createdGroup = null;

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.isCreative())
                createdGroup = groupManager.createGroup(groupName);

            if (createdGroup != null)
                return createdGroup;
        }

        return null;
    }

    /**
     * Removes the specified group, if it is present.
     *
     * @throws EntityException
     *          - representing the exception which prohibited removal
     */
    public void removeGroup(Group group) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            if (groupManager.getGroup(group.getName()) == null)
                continue;

            if (groupManager.isReadOnly(group))
                continue;

            groupManager.removeGroup(group);
            break;
        }
    }

    public void addMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            throw new IllegalArgumentException("Can't add membership for null group");
        GroupManager groupManager = getMatchingGroupManager(group);
        groupManager.addMembership(group, user);
    }

    /**
     * Check whether this manager has a record of membership between the
     * argued user and group.
     *
     * @throws com.atlassian.user.impl.RepositoryException
     *
     */
    public boolean hasMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            return false;
        GroupManager groupManager = getMatchingGroupManager(group);
        return groupManager.hasMembership(group, user);
    }

    public void removeMembership(Group group, User user) throws EntityException
    {
        if (group == null)
            throw new IllegalArgumentException("Can't remove membership for null group");
        GroupManager groupManager = getMatchingGroupManager(group);
        groupManager.removeMembership(group, user);
    }

    /**
     * @return true if users from other systems can be granted membership in the underlying {@link com.atlassian.user.repository.RepositoryIdentifier},
     *         otherwise false.
     * @throws EntityException
     *
     */
    public boolean supportsExternalMembership() throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.supportsExternalMembership())
                return true;
        }

        return false;
    }

    public boolean isReadOnly(Group group) throws EntityException
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.getGroup(group.getName()) != null)
                return groupManager.isReadOnly(group);
        }

        return false;
    }

    /**
     * @return the {@link com.atlassian.user.repository.RepositoryIdentifier} which is managed by this instance.
     */
    public RepositoryIdentifier getIdentifier()
    {
        ArrayList repositories = new ArrayList();

        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();
            RepositoryIdentifier repo = groupManager.getIdentifier();
            repositories.add(repo);
        }

        return new DelegatingRepository(repositories);
    }

    /**
     * Don't use for now. This is very expensive for large LDAP instances. todo: get user/group entities to store which repository they came from
     *
     * @return the {@link RepositoryIdentifier} in which the entity is stored, otherwise null.
     * @throws EntityException
     *
     */
    public RepositoryIdentifier getRepository(Entity entity) throws EntityException
    {
        if (!(entity instanceof Group))
            return null;

        GroupManager groupManager = getMatchingGroupManager((Group) entity);

        return groupManager.getIdentifier();
    }

    /**
     * Used to detemine whether an entity can be created (eg, can call {@link UserManager#createUser(String)} or
     * {@link GroupManager#createGroup(String)}
     *
     * @return true to indicate that {@link Entity} objects can be created by this manager, or false to indicate
     *         not.
     */
    public boolean isCreative()
    {
        Iterator iter = getGroupManagersIterator();

        while (iter.hasNext())
        {
            GroupManager groupManager = (GroupManager) iter.next();

            if (groupManager.isCreative())
                return true;
        }
        return false;
    }

    /**
     * Returns an iterator of {@link GroupManager} objects, in the order they
     * are to be used for this delegating manager.
     */
    protected abstract Iterator getGroupManagersIterator();

    private GroupManager getManagerForGroup(Group group) throws EntityException
    {
        for (Iterator it = getGroupManagersIterator(); it.hasNext();)
        {
            GroupManager groupManager = (GroupManager) it.next();
            if (groupManager.getGroup(group.getName()) != null)
                return groupManager;
        }

        return null;
    }
}
